﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Data;

using iTextSharp;
using iTextSharp.text;
using iTextSharp.text.pdf;
using iTextSharp.text.factories;

using System.IO;
using System.Collections;
using System.Windows.Forms;

namespace PDFUtilities
{
    class EncryptPDFHelper
    {
        public static string EncryptSinglePDF(string InputFile, string OutputFile, string Password, EncryptArgs encryptargs)
        {
            string err = "";

            try
            {
                //                using (FileStream input = new FileStream(InputFile, FileMode.Open, FileAccess.Read, FileShare.Read))
                //{
                PdfReader reader = null;

                if (!System.IO.Directory.Exists(System.IO.Path.GetDirectoryName(OutputFile)))
                {
                    try
                    {
                        System.IO.Directory.CreateDirectory(System.IO.Path.GetDirectoryName(OutputFile));
                    }
                    catch (Exception exd)
                    {
                        err += "Error. Could not Create Directory for File" + " : " + System.IO.Path.GetFileName(InputFile) + "\r\n" + exd.Message + "\r\n";
                        return err;
                    }
                }


                try
                {
                    //  reader = new PdfReader(input);

                    using (Stream input = new FileStream(InputFile, FileMode.Open, FileAccess.Read, FileShare.Read))
                    {
                        try
                        {
                            if (!String.IsNullOrEmpty(Password))
                            {
                                reader = new PdfReader(input, Encoding.Default.GetBytes(Password));
                                //reader = new PdfReader(input, Encoding.UTF8.GetBytes(Password));

                            }
                            else
                            {
                                reader = new PdfReader(input);
                            }                            
                        }
                        catch (iTextSharp.text.pdf.BadPasswordException ex)
                        {
                            if (reader != null)
                            {
                                reader.Close();
                            }

                            if (input != null)
                            {
                                input.Close();
                            }


                            err += "Error. Wrong User Password for File" + " : " + System.IO.Path.GetFileName(InputFile);
                            return err;
                        }
                    }
                    

                    string userp = encryptargs.txtUserPassword;
                    if (userp == "")
                    {
                        userp = null;
                    }

                    string ownerp = encryptargs.txtOwnerPassword;
                    if (ownerp == "")
                    {
                        ownerp = null;
                    }                                        

                    int permissions = 0;
                    if (encryptargs.chkAnnotations)
                    {
                        permissions |= PdfWriter.ALLOW_MODIFY_ANNOTATIONS;
                    }

                    if (encryptargs.chkAssembly)
                    {
                        permissions |= PdfWriter.ALLOW_ASSEMBLY;
                    }

                    if (encryptargs.chkCopy)
                    {
                        permissions |= PdfWriter.ALLOW_COPY;
                    }

                    if (encryptargs.chkFormFill)
                    {
                        permissions |= PdfWriter.ALLOW_FILL_IN;
                    }

                    if (encryptargs.chkHighPrinting)
                    {
                        permissions |= PdfWriter.ALLOW_PRINTING;
                    }

                    if (encryptargs.chkLowPrinting)
                    {
                        permissions |= PdfWriter.ALLOW_DEGRADED_PRINTING;
                    }

                    if (encryptargs.chkModifyContents)
                    {
                        permissions |= PdfWriter.ALLOW_MODIFY_CONTENTS;
                    }

                    if (encryptargs.chkScreenReaders)
                    {
                        permissions |= PdfWriter.ALLOW_SCREENREADERS;
                    }

                    bool strength = encryptargs.rd128Bits;

                    //if (ownerp == null && userp == null)
                    //{
                    using (MemoryStream memoryStream = new MemoryStream())
                    {
                        PdfStamper stamper = new PdfStamper(reader, memoryStream);                        

                        if (ownerp != null || userp != null)
                        {
                            stamper.SetEncryption(userp != null ? Encoding.ASCII.GetBytes(userp) : null, ownerp != null ? Encoding.ASCII.GetBytes(ownerp) : null,
                                permissions, strength);
                        }


                        /*
                                                public byte[] EncryptPdf(byte[] src) {
                          PdfReader reader = new PdfReader(src);
                          using (MemoryStream ms = new MemoryStream()) {
                            using (PdfStamper stamper = new PdfStamper(reader, ms)) {
                              stamper.SetEncryption(
                                USER, OWNER, 
                                PdfWriter.ALLOW_PRINTING, 
                                PdfWriter.ENCRYPTION_AES_128 | PdfWriter.DO_NOT_ENCRYPT_METADATA
                              );
                            }
                            return ms.ToArray();
                          }
                        }*/
                        stamper.Close();
                        reader.Close();

                        File.WriteAllBytes(OutputFile, memoryStream.ToArray());                        

                        return err;
                    }

                    /*
                    using (Stream output = new FileStream(OutputFile, FileMode.Create, FileAccess.Write, FileShare.None))
                    {
                        int permissions = 0;
                        if (encryptargs.chkAnnotations.Checked)
                        {
                            permissions |= PdfWriter.ALLOW_MODIFY_ANNOTATIONS;
                        }

                        if (encryptargs.chkAssembly.Checked)
                        {
                            permissions |= PdfWriter.ALLOW_ASSEMBLY;
                        }

                        if (encryptargs.chkCopy.Checked)
                        {
                            permissions |= PdfWriter.ALLOW_COPY;
                        }

                        if (encryptargs.chkFormFill.Checked)
                        {
                            permissions |= PdfWriter.ALLOW_FILL_IN;
                        }

                        if (encryptargs.chkHighPrinting.Checked)
                        {
                            permissions |= PdfWriter.ALLOW_PRINTING;
                        }

                        if (encryptargs.chkLowPrinting.Checked)
                        {
                            permissions |= PdfWriter.ALLOW_DEGRADED_PRINTING;
                        }

                        if (encryptargs.chkModifyContents.Checked)
                        {
                            permissions |= PdfWriter.ALLOW_MODIFY_CONTENTS;
                        }

                        if (encryptargs.chkScreenReaders.Checked)
                        {
                            permissions |= PdfWriter.ALLOW_SCREENREADERS;
                        }

                        bool strength = encryptargs.rd128Bits.Checked;

                        //if (SetDocInfo)
                        //{                                               

                        if (dict.Count > 0)
                        {
                            //PdfEncryptor.Encrypt(reader, output, strength, userp, ownerp, permissions, dict);

                            PdfEncryptor.Encrypt(reader, output, Encoding.Default.GetBytes(userp), Encoding.Default.GetBytes(ownerp),
                                permissions, strength);
                        }
                        else
                        {
                            //PdfEncryptor.Encrypt(reader, output, strength, userp, ownerp, permissions);

                            PdfEncryptor.Encrypt(reader, output, Encoding.Default.GetBytes(userp), Encoding.Default.GetBytes(ownerp),
                                permissions, strength);
                        }
                    }
                    */

                    
                }
                catch (Exception exm)
                {
                    err += "An error occured while protecting PDF File" + " : " + System.IO.Path.GetFileName(InputFile) + "\r\n" + exm.Message + "\r\n";
                }
                finally
                {
                    if (reader != null)
                    {
                        reader.Close();
                    }
                }
            }
            catch (Exception ex)
            {
                err += "An error occured while protecting PDF File" + " : " + System.IO.Path.GetFileName(InputFile) + "\r\n" + ex.Message + "\r\n";
            }

            return err;
        }
    }

    public class EncryptArgs
    {
        public string cmbOutputDir = "";
        public bool chkCopy = false;
        public bool chkFormFill = false;
        public bool chkHighPrinting = false;
        public bool chkLowPrinting = false;
        public bool chkModifyContents = false;
        public bool chkScreenReaders = false;
        public bool chkAssembly = false;
        public bool chkAnnotations = false;
        public bool rd128Bits = false;
        public string txtUserPassword = "";
        public string txtOwnerPassword = "";
    }
}
