﻿using System;
using System.Collections.Generic;
using System.Text;
using iTextSharp.text.pdf;
using iTextSharp.text.pdf.parser;
using System.Data;
using System.Windows.Forms;
using System.Drawing;
using System.IO;

namespace PDFUtilities
{
    class PDFImageExtractor
    {
        public static String RESULT = "Img{0}.{1}";
        public static List<Image> lstAddImages = new List<Image>();

        public static string ExtractImages(string filepath, string outputDir,string password, bool allPages,
            string pageRanges,
            bool pagesEvery, int pagesEveryValue,
            bool oddPages, bool evenPages,
            bool containingText, string textToContain)
        {
            string err = "";

            try
            {
                PdfReader reader = null;

                if (String.IsNullOrEmpty(password))
                {
                    reader = new PdfReader(filepath);
                }
                else
                {
                    reader = new PdfReader(filepath, Encoding.Default.GetBytes(password));
                }

                RESULT = System.IO.Path.Combine(outputDir, System.IO.Path.GetFileName(filepath) + ".extracted_images");

                if (!System.IO.Directory.Exists(RESULT))
                {
                    System.IO.Directory.CreateDirectory(RESULT);
                }

                RESULT += "\\Img[PAGENUM]-[NUM].jpg";

                //RESULT+="\\Img[FILENAME] - [PAGENUM]-[NUM].jpg";

                iTextSharp.text.pdf.parser.PdfReaderContentParser parser = new iTextSharp.text.pdf.parser.PdfReaderContentParser(reader);
                MyImageRenderListener listener = new MyImageRenderListener(RESULT);
                
                listener.PdfFile = System.IO.Path.GetFileName(filepath);

                for (int i = 1; i <= reader.NumberOfPages; i++)
                {                    
                    try
                    {
                        if (ShouldAddPage(i, reader, allPages,
             pageRanges,
            pagesEvery, pagesEveryValue,
             oddPages,  evenPages, 
             containingText, textToContain))
                        {
                            listener.pagenumber = i;

                            parser.ProcessContent(i, listener);
                        }
                    }
                    catch (Exception exi)
                    {
                        err += exi.Message + "\r\n";
                    }
                }
            }
            catch (Exception ex)
            {
                err += ex.Message + "\r\n";
            }

            return err;
        }

        #region Methods
        #region Public Methods
        /// <summary>Checks whether a specified page of a PDF file contains images.</summary>
        /// <returns>True if the page contains at least one image; false otherwise.</returns>
        public static bool PageContainsImages(string filename, int pageNumber)
        {
            PdfReader reader = null;
            try
            {
                //using (var reader = new PdfReader(filename))
                //{

                //var parser = new PdfReaderContentParser(reader);                                     

                PdfReaderContentParser parser = new PdfReaderContentParser(reader);
                ImageRenderListener listener = null;
                parser.ProcessContent(pageNumber, (listener = new ImageRenderListener()));
                return listener.Images.Count > 0;

                //}
            }
            finally
            {
                if (reader != null)
                {
                    reader.Close();
                }
            }
        }
        /// <summary>Extracts all images (of types that iTextSharp knows how to decode) from a PDF file.</summary>
        public static Dictionary<string, System.Drawing.Image> ExtractImages(string filename)
        {
            var images = new Dictionary<string, System.Drawing.Image>();
            //3using (var reader = new PdfReader(filename))
            PdfReader reader = null;

            try
            {
                reader = new PdfReader(filename);

                var parser = new PdfReaderContentParser(reader);
                ImageRenderListener listener = null;
                for (var i = 1; i <= reader.NumberOfPages; i++)
                {
                    parser.ProcessContent(i, (listener = new ImageRenderListener()));
                    var index = 1;
                    if (listener.Images.Count > 0)
                    {
                        Console.WriteLine("Found {0} images on page {1}.", listener.Images.Count, i);
                        foreach (var pair in listener.Images)
                        {
                            images.Add(string.Format("{0}_Page_{1}_Image_{2}{3}",
                                Path.GetFileNameWithoutExtension(filename), i.ToString("D4"), index.ToString("D4"), pair.Value), pair.Key);
                            index++;
                        }
                    }
                }
                return images;
            }
            finally
            {
                if (reader != null)
                {
                    reader.Close();
                }
            }
        }
        /// <summary>Extracts all images (of types that iTextSharp knows how to decode)
        /// from a specified page of a PDF file.</summary>
        /// <returns>Returns a generic <see cref="Dictionary&lt;string, System.Drawing.Image&gt;"/>,
        /// where the key is a suggested file name, in the format: PDF filename without extension,
        /// page number and image index in the page.</returns>
        public static Dictionary<string, System.Drawing.Image> ExtractImages(string filename, int pageNumber)
        {
            Dictionary<string, System.Drawing.Image> images = new Dictionary<string, System.Drawing.Image>();
            PdfReader reader = new PdfReader(filename);
            PdfReaderContentParser parser = new PdfReaderContentParser(reader);
            ImageRenderListener listener = null;
            parser.ProcessContent(pageNumber, (listener = new ImageRenderListener()));
            int index = 1;
            if (listener.Images.Count > 0)
            {
                Console.WriteLine("Found {0} images on page {1}.", listener.Images.Count, pageNumber);
                foreach (KeyValuePair<System.Drawing.Image, string> pair in listener.Images)
                {
                    images.Add(string.Format("{0}_Page_{1}_Image_{2}{3}",
                        Path.GetFileNameWithoutExtension(filename), pageNumber.ToString("D4"), index.ToString("D4"), pair.Value), pair.Key);
                    index++;
                }
            }
            return images;
        }
        #endregion Public Methods
        #endregion Methods

        public static bool ShouldAddPage(int pagenum, 
            PdfReader reader,
            bool allPages,
            string pageRanges,
            bool pagesEvery, int pagesEveryValue,
            bool oddPages, bool evenPages, 
            bool containingText, string textToContain)
            // all pages= return all pages
            // page range => check if odd check if even check if containing text
            // pages every => check pages every check odd check even check containing text
            // containing text => check containing text
            // odd => check odd
            // even => check even
        {
            bool contain = false;

            if (allPages)
            {
                contain=true;
            }

            bool add = false;            

            if (pageRanges.Trim() != String.Empty)
            {
                string[] ranges = pageRanges.Split(new string[] { "," }, StringSplitOptions.RemoveEmptyEntries);

                for (int k = 0; k < ranges.Length; k++)
                {
                    int ifrom = 0;
                    int ito = 0;

                    if (ranges != null)
                    {
                        if (ranges[k].IndexOf("-") >= 0)
                        {
                            string[] ranges2 = ranges[k].Split(new string[] { "-" }, StringSplitOptions.RemoveEmptyEntries);

                            ifrom = int.Parse(ranges2[0]);
                            ito = int.Parse(ranges2[1]);
                        }
                        else
                        {
                            ifrom = int.Parse(ranges[k]);
                            ito = ifrom;
                        }
                    }

                    if (pagenum < ifrom || pagenum > ito)
                    {
                        
                    }
                    else
                    {
                        contain=true;
                    }
                }
            }

            if (!contain) return false;

            if (pagesEvery)
            {
                int startPage = -1;

                if (allPages)
                {
                    startPage = 1;
                }
                else
                {
                    string[] ranges = pageRanges.Split(new string[] { "," }, StringSplitOptions.RemoveEmptyEntries);

                    for (int k = 0; k < ranges.Length; k++)
                    {
                        string[] range = ranges[k].Split(new string[] { "-" }, StringSplitOptions.RemoveEmptyEntries);

                        int ifrom = int.Parse(range[0]);
                        int ito = int.Parse(range[1]);

                        if (pagenum < ifrom || pagenum > ito)
                        {

                        }
                        else
                        {
                            startPage = ifrom;
                        }
                    }
                }

                if ((pagenum - startPage) % pagesEveryValue == 0)
                {
                    
                }
                else
                {
                    return false;
                }

            }

            if (containingText && reader != null)
            {
                if (iTextSharp.text.pdf.parser.PdfTextExtractor.GetTextFromPage(reader, pagenum).Contains(
                    textToContain))
                {

                }
                else
                {
                    return false;
                }
            }

            if (oddPages)
            {
                if (pagenum % 2 != 0)
                {

                }
                else
                {
                    return false;
                }
            }

            if (evenPages)
            {
                if (pagenum % 2 == 0)
                {

                }
                else
                {
                    return false;
                }
            }

            return true;
        }        
    }

        class NewMyImageRenderListener : iTextSharp.text.pdf.parser.IRenderListener
        {
            /** The new document to which we've added a border rectangle. */
            protected String path = "";                       

            public string PdfFile = "";
            public int pagenumber = 0;

            private int ImageNumber = 1;

            /**
             * Creates a RenderListener that will look for images.
             */
            public NewMyImageRenderListener(String path)
            {
                this.path = path;
            }

            /**
             * @see com.itextpdf.text.pdf.parser.RenderListener#beginTextBlock()
             */
            public void BeginTextBlock()
            {
            }

            /**
             * @see com.itextpdf.text.pdf.parser.RenderListener#endTextBlock()
             */
            public void EndTextBlock()
            {
            }

            /**
             * @see com.itextpdf.text.pdf.parser.RenderListener#renderImage(
             *     com.itextpdf.text.pdf.parser.ImageRenderInfo)
             */
            public void RenderImage(iTextSharp.text.pdf.parser.ImageRenderInfo renderInfo)
            {
                try
                {
                    String filename = path;

                    PdfImageObject image = renderInfo.GetImage();
                    PdfName filter = (PdfName)image.Get(PdfName.FILTER);

                    if (image == null) return;

                    if (filter != null)
                    {
                    System.Drawing.Image drawingImage = image.GetDrawingImage();

                        if (drawingImage==null) return;

                    //3string extension = ".";

                    string extension = "";

                    if (filter == PdfName.DCTDECODE)
                    {
                        extension += PdfImageObject.ImageBytesType.JPG.FileExtension;
                    }
                    else if (filter == PdfName.JPXDECODE)
                    {
                        extension += PdfImageObject.ImageBytesType.JP2.FileExtension;
                    }
                    else if (filter == PdfName.FLATEDECODE)
                    {
                        extension += PdfImageObject.ImageBytesType.PNG.FileExtension;
                    }
                    else if (filter == PdfName.LZWDECODE)
                    {
                        extension += PdfImageObject.ImageBytesType.CCITT.FileExtension;
                    }
                    /* Rather than struggle with the image stream and try to figure out how to handle
                     * BitMapData scan lines in various formats (like virtually every sample I’ve found
                     * online), use the PdfImageObject.GetDrawingImage() method, which does the work for us. */
                    //3this.Images.Add(drawingImage, extension);
                    
                    /*
                    System.IO.MemoryStream os;
                    iTextSharp.text.pdf.parser.PdfImageObject image = renderInfo.GetImage();
                    if (image == null) return;
                    */
                    filename = filename.Replace("[PAGENUM]", pagenumber.ToString());
                    filename = filename.Replace("[NUM]", ImageNumber.ToString());
                    filename = filename.Replace("[FILENAME]", PdfFile);
                    //3filename = filename.Replace("[FILETYPE]", image.GetFileType());

                    filename = filename.Replace("[FILETYPE]", extension);

                    //filename = String.Format(path, renderInfo.GetRef().Number, image.GetFileType());

                    ImageNumber++;

                    /*
                    os = new System.IO.MemoryStream(); //filename,System.IO.FileMode.Create);
                    os.Write(image.GetImageAsBytes(), 0, image.GetImageAsBytes().Length);
                    os.Flush();

                    os.Position = 0;

                    Image img = Image.FromStream(os);
                    */
                    
                        //3ImagesExtractorHelper.Convert(img, filename, add_to_one_pdf);

                    drawingImage.Save(filename);                   
                    

                    //3os.Close();
                    }
                }
                catch (Exception e)
                {
                    Console.WriteLine(e.Message);
                }
            }

            /**
             * @see com.itextpdf.text.pdf.parser.RenderListener#renderText(
             *     com.itextpdf.text.pdf.parser.TextRenderInfo)
             */
            public void RenderText(iTextSharp.text.pdf.parser.TextRenderInfo renderInfo)
            {
            }
        }

        class MyImageRenderListener : iTextSharp.text.pdf.parser.IRenderListener
        {
            /** The new document to which we've added a border rectangle. */
            protected String path = "";
            public bool add_to_one_pdf = false;
            public bool for_preview_page_extract = false;

            public string PdfFile = "";
            public int pagenumber = 0;

            private int ImageNumber = 1;

            /**
             * Creates a RenderListener that will look for images.
             */
            public MyImageRenderListener(String path)
            {
                this.path = path;
            }

            /**
             * @see com.itextpdf.text.pdf.parser.RenderListener#beginTextBlock()
             */
            public void BeginTextBlock()
            {
            }

            /**
             * @see com.itextpdf.text.pdf.parser.RenderListener#endTextBlock()
             */
            public void EndTextBlock()
            {
            }

            /**
             * @see com.itextpdf.text.pdf.parser.RenderListener#renderImage(
             *     com.itextpdf.text.pdf.parser.ImageRenderInfo)
             */
            public void RenderImage(iTextSharp.text.pdf.parser.ImageRenderInfo renderInfo)
            {
                try
                {
                    String filename = path;
                    System.IO.MemoryStream os;
                    iTextSharp.text.pdf.parser.PdfImageObject image = renderInfo.GetImage();
                    if (image == null) return;

                    filename = filename.Replace("[PAGENUM]", pagenumber.ToString());
                    filename = filename.Replace("[NUM]", ImageNumber.ToString());
                    filename = filename.Replace("[FILENAME]", PdfFile);
                    filename = filename.Replace("[FILETYPE]", image.GetFileType());

                    //filename = String.Format(path, renderInfo.GetRef().Number, image.GetFileType());

                    ImageNumber++;

                    os = new System.IO.MemoryStream(); //filename,System.IO.FileMode.Create);
                    os.Write(image.GetImageAsBytes(), 0, image.GetImageAsBytes().Length);
                    os.Flush();

                    os.Position = 0;

                    Image img = Image.FromStream(os);

                    img.Save(filename);

                    os.Close();
                }
                catch (Exception e)
                {
                    Console.WriteLine(e.Message);
                }
            }

            /**
             * @see com.itextpdf.text.pdf.parser.RenderListener#renderText(
             *     com.itextpdf.text.pdf.parser.TextRenderInfo)
             */
            public void RenderText(iTextSharp.text.pdf.parser.TextRenderInfo renderInfo)
            {
            }
        }
        
    public class ImageRenderListener : IRenderListener
        {
            #region Fields
            Dictionary<System.Drawing.Image, string> images = new Dictionary<System.Drawing.Image, string>();
            #endregion Fields
            #region Properties
            public Dictionary<System.Drawing.Image, string> Images
            {
                get { return images; }
            }
            #endregion Properties
            
            public void BeginTextBlock() { }
            public void EndTextBlock() { }
            public void RenderImage(ImageRenderInfo renderInfo)
            {
                PdfImageObject image = renderInfo.GetImage();
                PdfName filter = (PdfName)image.Get(PdfName.FILTER);

                //int width = Convert.ToInt32(image.Get(PdfName.WIDTH).ToString());
                //int bitsPerComponent = Convert.ToInt32(image.Get(PdfName.BITSPERCOMPONENT).ToString());
                //string subtype = image.Get(PdfName.SUBTYPE).ToString();
                //int height = Convert.ToInt32(image.Get(PdfName.HEIGHT).ToString());
                //int length = Convert.ToInt32(image.Get(PdfName.LENGTH).ToString());
                //string colorSpace = image.Get(PdfName.COLORSPACE).ToString();
                /* It appears to be safe to assume that when filter == null, PdfImageObject
                 * does not know how to decode the image to a System.Drawing.Image.
                 *
                 * Uncomment the code above to verify, but when I’ve seen this happen,
                 * width, height and bits per component all equal zero as well. */
                if (filter != null)
                {
                    System.Drawing.Image drawingImage = image.GetDrawingImage();
                    string extension = ".";
                    if (filter == PdfName.DCTDECODE)
                    {
                        extension += PdfImageObject.ImageBytesType.JPG.FileExtension;
                    }
                    else if (filter == PdfName.JPXDECODE)
                    {
                        extension += PdfImageObject.ImageBytesType.JP2.FileExtension;
                    }
                    else if (filter == PdfName.FLATEDECODE)
                    {
                        extension += PdfImageObject.ImageBytesType.PNG.FileExtension;
                    }
                    else if (filter == PdfName.LZWDECODE)
                    {
                        extension += PdfImageObject.ImageBytesType.CCITT.FileExtension;
                    }
                    /* Rather than struggle with the image stream and try to figure out how to handle
                     * BitMapData scan lines in various formats (like virtually every sample I’ve found
                     * online), use the PdfImageObject.GetDrawingImage() method, which does the work for us. */
                    this.Images.Add(drawingImage, extension);
                }
            }
            public void RenderText(TextRenderInfo renderInfo) { }
    }    
}
