﻿using System;
using System.Collections.Generic;
using System.Text;
using iTextSharp.text.pdf;
using iTextSharp.text.pdf.parser;
using iTextSharp.text;
using System.Data;
using System.IO;

namespace PDFUtilitiesNew
{
    class PDFExtractor
    {
        public static string ExtractPDFPages(string password, string filepath, string outputFilepath,
            string pageRanges,
            bool pagesEvery, int pagesEveryValue,
            bool oddPages, bool evenPages,
            bool containingText, string textToContain)
        {
            try
            {
                PdfReader reader = null;

                if (String.IsNullOrEmpty(password))
                {
                    reader = new PdfReader(filepath);
                }
                else
                {
                    reader = new PdfReader(filepath, Encoding.Default.GetBytes(password));
                }

                using (MemoryStream msCopy = new MemoryStream())
                {
                    using (Document docCopy = new Document())
                    {
                        using (PdfCopy copy = new PdfCopy(docCopy, msCopy))
                        {
                            docCopy.Open();

                            //for (int pageNum = 1; pageNum <= reader.NumberOfPages - 1; pageNum ++)
                            for (int pageNum = 1; pageNum <= reader.NumberOfPages; pageNum++)
                            {
                                if (ShouldIncludePage(pageNum,
                                    reader, pageRanges, pagesEvery, pagesEveryValue, oddPages, evenPages, containingText, textToContain))
                                {
                                    copy.AddPage(copy.GetImportedPage(reader, pageNum));
                                }
                            }

                            docCopy.Close();
                        }
                    }

                    System.IO.File.WriteAllBytes(outputFilepath, msCopy.ToArray());

                    return "";
                }
            }
            catch (Exception ex)
            {
                return "Error could not extract PDF pages of document " + System.IO.Path.GetFileName(filepath) + " " + ex.Message;
            }
        }

        private static bool ShouldIncludePage(int pagenum,
            PdfReader reader,            
            string pageRanges,
            bool pagesEvery, int pagesEveryValue,
            bool oddPages, bool evenPages,
            bool containingText, string textToContain          
            )
        // all pages= return all pages
        // page range => check if odd check if even check if containing text
        // pages every => check pages every check odd check even check containing text
        // containing text => check containing text
        // odd => check odd
        // even => check even
        {

            // returns 1 => include page
            // return 2 => split page
            // return 0 => do not include page            

            bool contain = true;

            if (pageRanges.Trim() != String.Empty)
            {
                contain = false;

                string[] ranges = pageRanges.Split(new string[] { "," }, StringSplitOptions.RemoveEmptyEntries);

                for (int k = 0; k < ranges.Length; k++)
                {
                    int ifrom = 0;
                    int ito = 0;

                    if (ranges[k].IndexOf("-") >= 0)
                    {
                        string[] ranges2 = ranges[k].Split(new string[] { "-" }, StringSplitOptions.RemoveEmptyEntries);

                        ifrom = int.Parse(ranges2[0]);
                        ito = int.Parse(ranges2[1]);
                    }
                    else
                    {
                        ifrom = int.Parse(ranges[k]);
                        ito = ifrom;
                    }

                    if (pagenum < ifrom || pagenum > ito)
                    {

                    }
                    else
                    {
                        contain = true;
                    }
                }
            }

            if (!contain)
            {
                return false;
            }            

            if (pagesEvery)
            {
                contain = false;

                int startPage = 1;

                if (pageRanges != string.Empty)
                {
                    string[] ranges = pageRanges.Split(new string[] { "," }, StringSplitOptions.RemoveEmptyEntries);

                    for (int k = 0; k < ranges.Length; k++)
                    {
                        string[] range = ranges[k].Split(new string[] { "-" }, StringSplitOptions.RemoveEmptyEntries);

                        int ifrom = int.Parse(range[0]);
                        int ito = int.Parse(range[1]);

                        if (pagenum < ifrom || pagenum > ito)
                        {

                        }
                        else
                        {
                            startPage = ifrom;
                        }
                    }
                }

                if ((pagenum - startPage) % pagesEveryValue == 0)
                {
                    contain = true;
                }
                else
                {
                    
                }

            }

            if (!contain)
            {
                return false;
            }            

            if (containingText && reader != null)
            {
                contain = false;

                if (iTextSharp.text.pdf.parser.PdfTextExtractor.GetTextFromPage(reader, pagenum).Contains(
                    textToContain))
                {
                    contain = true;
                }
                else
                {

                }
            }

            if (!contain)
            {
                return false;
            }            

            if (oddPages)
            {
                contain = false;

                if (pagenum % 2 != 0)
                {
                    contain = true;
                }
                else
                {

                }
            }

            if (!contain)
            {
                return false;
            }            

            if (evenPages)
            {
                contain = false;

                if (pagenum % 2 == 0)
                {
                    contain = true;
                }
                else
                {

                }
            }

            return contain;
        }        
    }
}
