﻿using System;
using System.Collections.Generic;
using System.Text;
using iTextSharp.text.pdf;
using iTextSharp.text.pdf.parser;
using iTextSharp.text;
using System.Data;
using System.IO;

namespace PDFUtilitiesNew
{
    class PDFMetadataEditor
    {
        public static string EditPDFMetadata(string filepath, string outputFilepath, string password,
            bool clearAllMetadata,
            bool chkTitleNewValue,string titleNewValue,
            bool chkAuthorNewValue,string AuthorNewValue,
            bool chkSubjectNewValue,string SubjectNewValue,
            bool chkKeywordsNewValue,string KeywordsNewValue,
            bool chkCreatorNewValue,string CreatorNewValue,
            bool chkCreationDateNewValue,string CreationDateNewValue,
            bool chkModDateNewValue,string ModDateNewValue,
            bool wildcards,bool regexp,
            bool chkFindReplace,string find,string replace,
            List<string> customName,List<string> customValue)            
        {
            try
            {
                PdfReader reader = null;

                if (String.IsNullOrEmpty(password))
                {
                    reader = new PdfReader(filepath);
                }
                else
                {
                    reader = new PdfReader(filepath, Encoding.Default.GetBytes(password));
                }

                Dictionary<string, string> dict = new Dictionary<string, string>();
                //PdfDictionary dict = new PdfDictionary();

                if (chkAuthorNewValue)
                {                   
                    dict.Add("Author", AuthorNewValue);
                }                                              

                if (chkCreatorNewValue)
                {
                    dict.Add("Creator", CreatorNewValue);
                }
                
                if (chkKeywordsNewValue)
                {
                    dict.Add("Keywords", KeywordsNewValue);
                }
                
                if (chkModDateNewValue)
                {                    
                    dict.Add(PdfName.MODDATE.ToString(), new PdfDate(DateTime.Parse(ModDateNewValue,
                        System.Globalization.CultureInfo.InvariantCulture)).ToString());
                }
                
                if (chkCreationDateNewValue)
                {                    
                    dict.Add("CreationDate", new PdfDate(DateTime.Parse(CreationDateNewValue,
                        System.Globalization.CultureInfo.InvariantCulture)).ToString());                    
                }

                if (chkTitleNewValue)
                {
                    dict.Add("Title", titleNewValue);
                }
                
                if (chkSubjectNewValue)
                {
                    dict.Add("Subject", SubjectNewValue);
                }
                
                DateTime curdate = DateTime.Now;

                for (int m = 0; m < customName.Count; m++)
                {
                    if (customName[m] != string.Empty)
                    {                        
                        dict.Add(customName[m], customValue[m]);
                    }
                }

                foreach (KeyValuePair<string, string> kv in reader.Info)
                {
                    bool foundkey = false;

                    foreach (KeyValuePair<string, string> kv1 in dict)
                    {
                        if (kv1.Key.ToLower() == kv.Key.ToLower())
                        {
                            foundkey = true;
                            break;
                        }
                    }

                    if (!foundkey)
                    {
                        if (chkFindReplace)
                        {
                            string newval = GetFindReplace(kv.Key, reader.Info, wildcards, regexp, find, replace);

                            dict.Add(kv.Key, newval);

                        }
                        else
                        {
                            dict.Add(kv.Key, kv.Value);
                        }
                    }
                }

                using (MemoryStream memoryStream = new MemoryStream())
                {
                    PdfStamper stamper = new PdfStamper(reader, memoryStream);

                    if (clearAllMetadata)
                    {
                        dict = new Dictionary<string, string>();

                        foreach (KeyValuePair<string, string> kv in reader.Info)
                        {
                            dict.Add(kv.Key, null);
                        }

                        if (dict.Count > 0)
                        {
                            stamper.MoreInfo = dict;
                        }

                        using (MemoryStream msb = new MemoryStream())
                        {
                            iTextSharp.text.xml.xmp.XmpWriter xmpw = new iTextSharp.text.xml.xmp.XmpWriter(msb);
                            xmpw.Close();
                            stamper.XmpMetadata = msb.ToArray();

                        }                        
                    }
                    else if (!clearAllMetadata)
                    {
                        if (dict.Count > 0)
                        {
                            stamper.MoreInfo = dict;
                        }
                    }

                    stamper.Close();
                    reader.Close();

                    File.WriteAllBytes(outputFilepath, memoryStream.ToArray());
                }                                

                return "";
            }

            catch (Exception ex)
            {
                return "Error could not edit PDF Metadata " + System.IO.Path.GetFileName(filepath) + " " + ex.Message;
            }
        }

        private static string GetFindReplace(string field, Dictionary<string,string> info,bool wildcards, bool regexp, string find, string replace)
        {
            string curval = "";

            if (info.ContainsKey(field))
            {
                curval = info[field];
            }

            if (wildcards)
            {
                string sp = RegexEscape(find);

                sp = sp.Replace(@"\?", ".{1}").Replace(@"\*", ".*?\\b");

                System.Text.RegularExpressions.Regex regex = new System.Text.RegularExpressions.Regex(sp);

                curval=regex.Replace(curval, replace);

            }
            else if (regexp)
            {
                System.Text.RegularExpressions.Regex regex = new System.Text.RegularExpressions.Regex(find);

                curval=regex.Replace(curval, replace);
            }
            else
            {
                string sp = RegexEscape(find);

                System.Text.RegularExpressions.Regex regex = new System.Text.RegularExpressions.Regex(sp);

                curval=regex.Replace(curval, replace);
            }

            return curval;
        }

        private static string RegexEscape(string searchpattern)
        {
            return System.Text.RegularExpressions.Regex.Escape(searchpattern).Replace(@"\\\ ", " ");
        }
    }
}
