﻿using System;
using System.Collections.Generic;
using System.Text;
using iTextSharp.text.pdf;
using iTextSharp.text.pdf.parser;
using iTextSharp.text;
using System.Data;
using System.IO;

namespace PDFUtilitiesNew
{
    class PDFPageDeleter
    {
        public static string DeletePDFPages(string password, string filepath,string outputFilepath,
            bool emptyPages, string pageRanges,
            bool pagesEvery, int pagesEveryValue,
            bool oddPages, bool evenPages,
            bool containingText, string textToContain)
        {
            try
            {            
                PdfReader reader = null;

                if (String.IsNullOrEmpty(password))
                {
                    reader = new PdfReader(filepath);
                }
                else
                {
                    reader = new PdfReader(filepath, Encoding.Default.GetBytes(password));
                }                

                using (MemoryStream msCopy = new MemoryStream())
                {
                    using (Document docCopy = new Document())
                   {
                        using (PdfCopy copy = new PdfCopy(docCopy, msCopy))
                        {
                            docCopy.Open();

                            //for (int pageNum = 1; pageNum <= reader.NumberOfPages - 1; pageNum ++)
                            for (int pageNum = 1; pageNum <= reader.NumberOfPages; pageNum++)
                            {
                                if (!ShouldDeletePage(pageNum,
                                    reader,emptyPages,pageRanges,pagesEvery,pagesEveryValue,oddPages,evenPages,containingText,textToContain))
                                {
                                copy.AddPage(copy.GetImportedPage(reader, pageNum ));
                                }
                            }

                        docCopy.Close();
                        }
                    }               
           
                    System.IO.File.WriteAllBytes(outputFilepath,msCopy.ToArray());           

                    return "";
                }
            }
            catch (Exception ex)
            {
                return "Error could not delete PDF pages of document " + System.IO.Path.GetFileName(filepath) + " " + ex.Message;
            }
        }

        private static bool ShouldDeletePage(int pagenum,
            PdfReader reader,            
            bool emptyPages,
            string pageRanges,
            bool pagesEvery, int pagesEveryValue,
            bool oddPages, bool evenPages,
            bool containingText, string textToContain)
        // all pages= return all pages
        // page range => check if odd check if even check if containing text
        // pages every => check pages every check odd check even check containing text
        // containing text => check containing text
        // odd => check odd
        // even => check even
        {
            
            bool add = false;

            if (emptyPages)
            {
                if (EmptyPDFPageChecker.IsPDFPageEmpty(reader,pagenum))
                {
                    return true;
                }
            }

            if (pageRanges.Trim() != String.Empty)
            {
                string[] ranges = pageRanges.Split(new string[] { "," }, StringSplitOptions.RemoveEmptyEntries);

                for (int k = 0; k < ranges.Length; k++)
                {
                    int ifrom = 0;
                    int ito = 0;

                    if (ranges != null)
                    {
                        if (ranges[k].IndexOf("-") >= 0)
                        {
                            string[] ranges2 = ranges[k].Split(new string[] { "-" }, StringSplitOptions.RemoveEmptyEntries);

                            ifrom = int.Parse(ranges2[0]);
                            ito = int.Parse(ranges2[1]);
                        }
                        else
                        {
                            ifrom = int.Parse(ranges[k]);
                            ito = ifrom;
                        }
                    }                    

                    if (pagenum < ifrom || pagenum > ito)
                    {

                    }
                    else
                    {
                        return true;
                    }
                }
            }            

            if (pagesEvery)
            {
                int startPage = -1;
                
                startPage = 1;                                

                if ((pagenum - startPage) % pagesEveryValue == 0)
                {
                    return true;
                }
                else
                {
                    
                }

            }

            if (containingText && reader != null)
            {
                if (iTextSharp.text.pdf.parser.PdfTextExtractor.GetTextFromPage(reader, pagenum).Contains(
                    textToContain))
                {
                    return true;
                }
                else
                {
                    
                }
            }

            if (oddPages)
            {
                if (pagenum % 2 != 0)
                {
                    return true;
                }
                else
                {
                    
                }
            }

            if (evenPages)
            {
                if (pagenum % 2 == 0)
                {
                    return true;
                }
                else
                {
                    
                }
            }

            return false;
        }
    }
}
