﻿using System;
using System.Collections.Generic;
using System.Text;
using iTextSharp.text.pdf;
using iTextSharp.text.pdf.parser;
using iTextSharp.text;
using System.Data;
using System.IO;

namespace PDFUtilitiesNew
{
    class PDFRotator
    {
        public static string RotatePDFPages(string password, string filepath, string outputFilepath,
            string pageRanges,
            bool pagesEvery, int pagesEveryValue,
            bool oddPages, bool evenPages,
            bool containingText, string textToContain,int angle)
        {
            try
            {
                PdfReader reader = null;

                if (String.IsNullOrEmpty(password))
                {
                    reader = new PdfReader(filepath);
                }
                else
                {
                    reader = new PdfReader(filepath, Encoding.Default.GetBytes(password));
                }

                for (int n = 1; n <= reader.NumberOfPages; n++)
                {
                    if (ShouldRotatePage(n,reader,pageRanges,pagesEvery,pagesEveryValue,oddPages,evenPages,containingText,textToContain))
                    {
                        PdfDictionary page = reader.GetPageN(n);
                        PdfNumber rotate = page.GetAsNumber(PdfName.ROTATE);
                        int rotation = rotate == null ? angle : (int.Parse(rotate.ToString()) + angle) % 360;

                        page.Put(PdfName.ROTATE, new PdfNumber(rotation));
                    }
                }

                PdfStamper stamper = new PdfStamper(reader, new System.IO.FileStream(outputFilepath,FileMode.Create));
                stamper.Close();
                reader.Close();

                return "";
            }

            catch (Exception ex)
            {
                return "Error could not rotate PDF pages of document " + System.IO.Path.GetFileName(filepath) + " " + ex.Message;
            }
        }

        private static bool ShouldRotatePage(int pagenum,
            PdfReader reader,            
            string pageRanges,
            bool pagesEvery, int pagesEveryValue,
            bool oddPages, bool evenPages,
            bool containingText, string textToContain)
        // all pages= return all pages
        // page range => check if odd check if even check if containing text
        // pages every => check pages every check odd check even check containing text
        // containing text => check containing text
        // odd => check odd
        // even => check even
        {

            bool contain = true;

            if (pageRanges.Trim() != String.Empty)
            {
                contain = false;

                string[] ranges = pageRanges.Split(new string[] { "," }, StringSplitOptions.RemoveEmptyEntries);

                for (int k = 0; k < ranges.Length; k++)
                {
                    int ifrom = 0;
                    int ito = 0;

                    if (ranges != null)
                    {
                        if (ranges[k].IndexOf("-") >= 0)
                        {
                            string[] ranges2 = ranges[k].Split(new string[] { "-" }, StringSplitOptions.RemoveEmptyEntries);

                            ifrom = int.Parse(ranges2[0]);
                            ito = int.Parse(ranges2[1]);
                        }
                        else
                        {
                            ifrom = int.Parse(ranges[k]);
                            ito = ifrom;
                        }
                    }                    

                    if (pagenum < ifrom || pagenum > ito)
                    {

                    }
                    else
                    {
                        return true;
                    }
                }
            }

            if (pagesEvery)
            {
                int startPage = -1;

                startPage = 1;

                if ((pagenum - startPage) % pagesEveryValue == 0)
                {
                    return true;
                }
                else
                {

                }

            }

            if (containingText && reader != null)
            {
                if (iTextSharp.text.pdf.parser.PdfTextExtractor.GetTextFromPage(reader, pagenum).Contains(
                    textToContain))
                {
                    return true;
                }
                else
                {

                }
            }

            if (oddPages)
            {
                if (pagenum % 2 != 0)
                {
                    return true;
                }
                else
                {

                }
            }

            if (evenPages)
            {
                if (pagenum % 2 == 0)
                {
                    return true;
                }
                else
                {

                }
            }

            return contain;
        }
    }
}
