﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using iTextSharp.text.pdf;
using iTextSharp.text.pdf.parser;
using iTextSharp.text;
using System.IO;

namespace PDFUtilitiesNew
{
    class PDFSplitter
    {
        public static string SplitPDFPages(string password, string filepath, string outputDir,
            bool emptyPages, int emptyPagesValue, string pageRanges,
            bool pagesEvery, int pagesEveryValue,
            bool oddPages, bool evenPages,
            bool containingText, string textToContain,bool splitBookmarks,int bookmarksLevel)
        {
            try
            {
                PdfReader reader = null;

                if (String.IsNullOrEmpty(password))
                {
                    reader = new PdfReader(filepath);
                }
                else
                {
                    reader = new PdfReader(filepath, Encoding.Default.GetBytes(password));
                }

                int splitnum = 1;

                string outputFilepath = System.IO.Path.Combine(outputDir, System.IO.Path.GetFileNameWithoutExtension(filepath) + "-split" + splitnum + ".pdf");

                bool finished = false;
                bool initialized = false;

                int pageNumStart = 1;

                while (!finished)
                {
                    try
                    {

                        int res = -1;

                        using (MemoryStream msCopy = new MemoryStream())
                        {
                            using (Document docCopy = new Document())
                            {
                                using (PdfCopy copy = new PdfCopy(docCopy, msCopy))
                                {
                                    docCopy.Open();

                                    for (int pageNum = pageNumStart; pageNum <= reader.NumberOfPages; pageNum++)
                                    {
                                        pageNumStart = pageNumStart + 1;

                                        res = ShouldSplitPage(pageNum,
                                            reader, emptyPages, emptyPagesValue, pageRanges, pagesEvery, pagesEveryValue, oddPages, evenPages, containingText, textToContain
                                            ,splitBookmarks,bookmarksLevel
                                            );

                                        if (res == 1 || (res == 2 && !initialized))
                                        {
                                            initialized = true;

                                            copy.AddPage(copy.GetImportedPage(reader, pageNum));
                                        }
                                        else if (res == 0)
                                        {

                                        }
                                        else if (res == 2)
                                        {
                                            splitnum = splitnum + 1;

                                            pageNumStart = pageNumStart - 1;

                                            initialized = false;

                                            break;
                                        }
                                    }

                                    docCopy.Close();
                                }
                            }


                            System.IO.File.WriteAllBytes(outputFilepath, msCopy.ToArray());

                            if (res == 2)
                            {
                                outputFilepath = System.IO.Path.Combine(outputDir, System.IO.Path.GetFileNameWithoutExtension(filepath) + "-split" + splitnum + ".pdf");
                            }
                            else if (res == -1)
                            {
                                finished = true;
                            }

                        }
                    }
                    catch {

                        finished = true;
                    }
                }


                return "";
            }
            catch (Exception ex)
            {
                return "Error could not split PDF document " + System.IO.Path.GetFileName(filepath) + " " + ex.Message;
            }
        }

        private static int ShouldSplitPage(int pagenum,
            PdfReader reader,
            bool emptyPages,
            int emptyPagesValue,
            string pageRanges,
            bool pagesEvery, int pagesEveryValue,
            bool oddPages, bool evenPages,
            bool containingText, string textToContain,
            bool splitBookmarks,
            int bookmarksLevel
            
            )
        // all pages= return all pages
        // page range => check if odd check if even check if containing text
        // pages every => check pages every check odd check even check containing text
        // containing text => check containing text
        // odd => check odd
        // even => check even
        {

            // returns 1 => include page
            // return 2 => split page
            // return 0 => do not include page            

            bool contain = true;

            if (pageRanges.Trim() != String.Empty)
            {
                contain = false;

                string[] ranges = pageRanges.Split(new string[] { "," }, StringSplitOptions.RemoveEmptyEntries);

                for (int k = 0; k < ranges.Length; k++)
                {
                    int ifrom = 0;
                    int ito = 0;

                    if (ranges != null)
                    {
                        if (ranges[k].IndexOf("-") >= 0)
                        {
                            string[] ranges2 = ranges[k].Split(new string[] { "-" }, StringSplitOptions.RemoveEmptyEntries);

                            ifrom = int.Parse(ranges2[0]);
                            ito = int.Parse(ranges2[1]);
                        }
                        else
                        {
                            ifrom = int.Parse(ranges[k]);
                            ito = ifrom;
                        }
                    }

                    if (pagenum < ifrom || pagenum > ito)
                    {
                        
                    }
                    else
                    {
                        if (pagenum==ifrom)
                        {
                            return 2; // split
                        }

                        contain = true;
                    }
                }
            }

            if (!contain)
            {
                return 0; // do not include
            }

            if (emptyPages)
            {
                bool isEmptyInARow = true;

                for (int m = pagenum-1; m >= 2 && m >= (pagenum-1 - emptyPagesValue + 1); m--)
                {
                    if (!(EmptyPDFPageChecker.IsPDFPageEmpty(reader, m)))
                    {
                        isEmptyInARow = false;
                    }
                }

                if (isEmptyInARow)
                {
                    if (EmptyPDFPageChecker.IsPDFPageEmpty(reader, pagenum))
                    {
                        return 0; // do not include consequent blank pages
                    }
                    else
                    {
                        return 2; // split after emptypagesvalue blank pages
                    }
                }
            }            

            if (pagesEvery)
            {
                int startPage = 1;                              
                
                if (pageRanges!=string.Empty)
                {
                    string[] ranges = pageRanges.Split(new string[] { "," }, StringSplitOptions.RemoveEmptyEntries);

                    for (int k = 0; k < ranges.Length; k++)
                    {
                        string[] range = ranges[k].Split(new string[] { "-" }, StringSplitOptions.RemoveEmptyEntries);

                        int ifrom = int.Parse(range[0]);
                        int ito = int.Parse(range[1]);

                        if (pagenum < ifrom || pagenum > ito)
                        {

                        }
                        else
                        {
                            startPage = ifrom;
                        }
                    }
                }

                if ((pagenum - startPage) % pagesEveryValue == 0)
                {
                    return 2;
                }
                else
                {

                }

            }

            if (containingText && reader != null)
            {
                if (iTextSharp.text.pdf.parser.PdfTextExtractor.GetTextFromPage(reader, pagenum).Contains(
                    textToContain))
                {
                    return 2;
                }
                else
                {

                }
            }

            if (oddPages)
            {
                if (pagenum % 2 != 0)
                {
                    return 2;
                }
                else
                {

                }
            }

            if (evenPages)
            {
                if (pagenum % 2 == 0)
                {
                    return 2;
                }
                else
                {

                }
            }

            if (splitBookmarks)
            {
                Bookmarks bookmarks = null;

                IList<Dictionary<String, Object>> bkmarks = new List<Dictionary<String, Object>>();

                if (reader != null)
                {
                    bkmarks = SimpleBookmark.GetBookmark(reader);
                }

                if (bkmarks != null)
                {
                    bookmarks = GetBookmarksTree(bkmarks);
                }

                if (bookmarks != null)
                {
                    List<string> bkmarks_short_text = new List<string>();
                    List<string> bkmarks_full_text = new List<string>();
                    List<int> bkmarks_level = new List<int>();

                    bkmarks_full_text.Clear();
                    bkmarks_short_text.Clear();
                    bkmarks_level.Clear();

                    int ibklevel = (int)bookmarksLevel;
                    List<int> bkpages = new List<int>();
                    GetBookmarkPagesAtLevel(bookmarks, 0, ibklevel, ref bkpages, "");

                    if (bkpages.Contains(pagenum))
                    {
                        return 2; // split
                    }
                }
            }

            return 1;//just include page
        }

        private static void GetBookmarkPagesAtLevel(Bookmarks bk, int level, int desired_level, ref List<int> bkpages, string bkmark_current_full_text)
        {
            if (level == desired_level)
            {
                bkpages.Add(bk.Page);
                //3bkmarks_short_text.Add(bk.Text);
                //3bkmarks_full_text.Add((bkmark_current_full_text != string.Empty ? bkmark_current_full_text + "." : "") + bk.Text);
                //3bkmarks_level.Add(level);
            }

            if (level < desired_level)
            {
                bkmark_current_full_text = (bkmark_current_full_text != string.Empty ? bkmark_current_full_text + "." : "") + bk.Text;

                for (int m = 0; m < bk.ChildBookmarks.Count; m++)
                {
                    GetBookmarkPagesAtLevel(bk.ChildBookmarks[m], level + 1, desired_level, ref bkpages, bkmark_current_full_text);
                }
            }
        }

        private static Bookmarks GetBookmarksTree(IList<Dictionary<String, Object>> bookmarks)
        {
            Bookmarks bk = new Bookmarks();

            GetBookmarksTreeLevel(bookmarks, ref bk);

            return bk;
        }

        private static void GetBookmarksTreeLevel(IList<Dictionary<String, Object>> bookmarks, ref Bookmarks bk)
        {
            int ipage = 0;

            for (int k = 0; k < bookmarks.Count; k++)
            {
                Bookmarks bknew = new Bookmarks();

                if (bookmarks[k].ContainsKey("Page"))
                {
                    string spage = bookmarks[k]["Page"].ToString();
                    int ipos = spage.IndexOf(" ");
                    ipage = int.Parse(spage.Substring(0, ipos));

                    if (!bk.ChildBookmarks.Contains(bknew))
                    {
                        bk.ChildBookmarks.Add(bknew);
                    }

                    bknew.Page = ipage;
                }

                if (bookmarks[k].ContainsKey("Title"))
                {
                    if (!bk.ChildBookmarks.Contains(bknew))
                    {
                        bk.ChildBookmarks.Add(bknew);
                    }

                    bknew.Text = bookmarks[k]["Title"].ToString();
                }

                if (bookmarks[k].ContainsKey("Kids"))
                {
                    IList<Dictionary<String, Object>> childbk = (IList<Dictionary<String, Object>>)bookmarks[k]["Kids"];
                    if (!bk.ChildBookmarks.Contains(bknew))
                    {
                        bk.ChildBookmarks.Add(bknew);
                    }

                    GetBookmarksTreeLevel(childbk, ref bknew);
                }
            }
        }
    }

    public class Bookmarks
    {
        public List<Bookmarks> ChildBookmarks = new List<Bookmarks>();
        public int Page = -1;
        public string Text = "";

        public Bookmarks()
        {

        }

        public Bookmarks this[int k]
        {
            get
            {
                return ChildBookmarks[k];
            }
            set
            {
                ChildBookmarks[k] = value;
            }
        }
    }
}
