﻿using System;
using System.Collections.Generic;
using System.Text;
using iTextSharp.text;
using iTextSharp.text.pdf;
using iTextSharp.text.pdf.parser;
using System.Data;
using System.IO;

namespace PDFUtilitiesNew
{
    class PDFWatermarker
    {
        public static string WatermarkPDF(string filepath, string outputFile, string password, 
            int r, int g, int b, float fontSize, float angle, string position, string watermarkText,string imageFile)
        {
            try
            {
                //imageFile = "";
                //fontSize = 24;
                //r = System.Drawing.Color.Red.R;
                //g = System.Drawing.Color.Red.G;
                //b = System.Drawing.Color.Red.B;
                //watermarkText = "This is a Test !";
                //angle = 180;

                PdfReader reader = null;

                if (String.IsNullOrEmpty(password))
                {
                    reader = new PdfReader(filepath);
                }
                else
                {
                    reader = new PdfReader(filepath, Encoding.Default.GetBytes(password));
                }

                string outputfp = outputFile;

                if (watermarkText != string.Empty && imageFile != string.Empty)
                {
                    outputFile = System.IO.Path.Combine(System.IO.Path.GetDirectoryName(outputFile), System.IO.Path.GetFileNameWithoutExtension(outputFile) + "[temp].pdf");
                }

                if (watermarkText != string.Empty)
                {
                    AddWatermark(reader, r, g, b, fontSize, angle, position, watermarkText, outputFile);
                }

                reader.Close();
                reader.Dispose();
                reader = null;

                if (watermarkText != string.Empty && imageFile != string.Empty)
                {
                    filepath = outputFile;

                    outputFile = outputfp;
                }

                if (imageFile != string.Empty)
                {
                    AddWatermarkImage(filepath, password,outputFile, imageFile,position,angle);

                    System.IO.File.Delete(filepath);
                }

                return "";
            }
            catch (Exception ex)
            {
                return "Error-Count not watermark PDF " + System.IO.Path.GetFileName(filepath) + " " + ex.Message;
            }
        }

        /// <summary>
        /// This method adds watermark text under pdf content
        /// </summary>
        /// <param name="pdfData">pdf content bytes</param>
        /// <param name="watermarkText">text to be shown as watermark</param>
        /// <param name="font">base font</param>
        /// <param name="fontSize">font sieze</param>
        /// <param name="angle">angle at which watermark needs to be shown</param>
        /// <param name="color">water mark color</param>
        /// <param name="realPageSize">pdf page size</param>
        private static void AddWaterMarkText(PdfContentByte pdfData, string watermarkText, BaseFont font, float fontSize, float angle, BaseColor color, string position,Rectangle realPageSize)
        {
            double radians = (angle * Math.PI) / 180;

            var gstate = new PdfGState { FillOpacity = 0.35f, StrokeOpacity = 0.3f };
            pdfData.SaveState();
            pdfData.SetGState(gstate);
            pdfData.SetColorFill(color);
            pdfData.BeginText();
            pdfData.SetFontAndSize(font, fontSize);

            float width = font.GetWidthPoint(watermarkText, fontSize);
            float height = font.GetAscentPoint(watermarkText, fontSize) - font.GetDescentPoint(watermarkText, fontSize);

            if (position == "Middle Center")
            {
                var x = (realPageSize.Left + realPageSize.Right) / 2;
                var y = (realPageSize.Bottom + realPageSize.Top) / 2;

                pdfData.ShowTextAligned(Element.ALIGN_CENTER, watermarkText, x, y, angle);
            }
            else if (position == "Middle Left")
            {
                //var x = realPageSize.Left + width / 2;

                var x = realPageSize.Left + (float)(Math.Cos(radians) * width) / 2 + 12;

                var y = (realPageSize.Bottom + realPageSize.Top) / 2;

                pdfData.ShowTextAligned(Element.ALIGN_CENTER, watermarkText, x, y, angle);
            }
            else if (position == "Middle Right")
            {
                //var x = realPageSize.Right - width / 2;

                var x = realPageSize.Right - (float)(Math.Cos(radians) * width) / 2 - 12;

                var y = (realPageSize.Bottom + realPageSize.Top) / 2;

                pdfData.ShowTextAligned(Element.ALIGN_CENTER, watermarkText, x, y, angle);
            }
            else if (position == "Top Left")
            {                
                

                //var x = realPageSize.Left + width/2;

                //var y = realPageSize.Top - height;

                var x = realPageSize.Left + (float)(Math.Cos(radians) * width) / 2 + 12;

                var y = realPageSize.Top - (float)(Math.Sin(radians) * width) / 2 -24;

                pdfData.ShowTextAligned(Element.ALIGN_CENTER, watermarkText, x, y, angle);
            }
            else if (position == "Top Right")
            {
                //var x = realPageSize.Right - width/2;

                //var y = realPageSize.Top - height;

                var x = realPageSize.Right - (float)(Math.Cos(radians) * width) / 2 - 12;

                var y = realPageSize.Top - (float)(Math.Sin(radians) * width) / 2 - 24;

                pdfData.ShowTextAligned(Element.ALIGN_CENTER, watermarkText, x, y, angle);
            }
            else if (position == "Top Center")
            {
                var x = (realPageSize.Left + realPageSize.Right) / 2;

                var y = realPageSize.Top - (float)(Math.Sin(radians) * width) / 2 - 24;

                //var y = realPageSize.Top - height;

                pdfData.ShowTextAligned(Element.ALIGN_CENTER, watermarkText, x, y, angle);
                
            }
            else if (position == "Bottom Left")
            {                

                //var x = realPageSize.Left + width / 2;

                var x = realPageSize.Left + (float)(Math.Cos(radians) * width) / 2 + 12;

                //var y = realPageSize.Bottom + height/2;

                var y = realPageSize.Bottom + (float)(Math.Sin(radians) * width) / 2 + 12;

                pdfData.ShowTextAligned(Element.ALIGN_CENTER, watermarkText, x, y, angle);
            }
            else if (position == "Bottom Right")
            {
                //var x = realPageSize.Right - width / 2;

                //var y = realPageSize.Bottom + height / 2;

                var x = realPageSize.Right - (float)(Math.Cos(radians) * width) / 2 - 12;

                var y = realPageSize.Bottom + (float)(Math.Sin(radians) * width) / 2 + 12;

                pdfData.ShowTextAligned(Element.ALIGN_CENTER, watermarkText, x, y, angle);
            }
            else if (position == "Bottom Center")
            {
                var x = (realPageSize.Left + realPageSize.Right) / 2;

                //var y = realPageSize.Bottom + height / 2;                

                var y = realPageSize.Bottom + (float)(Math.Sin(radians) * width) / 2 + 12;

                pdfData.ShowTextAligned(Element.ALIGN_CENTER, watermarkText, x, y, angle);

            }
                        
            pdfData.EndText();
            pdfData.RestoreState();
        }

        /*
        // <summary>
        /// This method calls another method to add watermark text for each page
        /// </summary>
        /// <param name="bytes">byte array of Pdf</param>
        /// <param name="baseFont">Base font</param>
        /// <param name="watermarkText">Text to be added as watermark</param>
        /// <returns>Pdf bytes array having watermark added</returns>
        private static byte[] AddWatermark(byte[] bytes, BaseFont baseFont, string watermarkText)
        {
            using (var ms = new MemoryStream(10 * 1024))
            {
                using (var reader = new PdfReader(bytes))
                using (var stamper = new PdfStamper(reader, ms))
                {
                    var pages = reader.NumberOfPages;
                    for (var i = 1; i <= pages; i++)
                    {
                        var dc = stamper.GetOverContent(i);
                        AddWaterMarkText(dc, watermarkText, baseFont, 50, 45, BaseColor.GRAY, reader.GetPageSizeWithRotation(i));
                    }
                    stamper.Close();
                }
                return ms.ToArray();
            }
        }
        */

        private static void AddWatermark(PdfReader reader, int r,int g,int b,float fontSize,float angle,string position,string watermarkText
            ,string outputFile
            )
        {
            using (var ms = new MemoryStream(10 * 1024))
            {                
                using (var stamper = new PdfStamper(reader, ms))
                {
                    var pages = reader.NumberOfPages;
                    for (var i = 1; i <= pages; i++)
                    {
                        var dc = stamper.GetOverContent(i);
                        AddWaterMarkText(
                            dc, 
                            watermarkText,
                            BaseFont.CreateFont(BaseFont.HELVETICA,Encoding.ASCII.EncodingName,false),
                            fontSize,
                            angle,
                            new BaseColor(r,g,b),position,
                            reader.GetPageSizeWithRotation(i));
                    }
                    stamper.Close();
                }

                System.IO.File.WriteAllBytes(outputFile, ms.ToArray());
            }
        }

        private static void AddWatermarkImage(string filepath, string password, string outputFile,string imageFile, string position,float angle)
        {
            System.Drawing.Image imgR = null;

            if (imageFile != string.Empty)
            {
                System.Drawing.Image imgR2 = System.Drawing.Image.FromFile(imageFile);
                imgR = (System.Drawing.Image)RotateImageHelper.RotateImage(imgR2, angle);

            }


            double radians = (angle * Math.PI) / 180;

            Document document = new Document();
            PdfReader reader = null;

            if (String.IsNullOrEmpty(password))
            {
                reader = new PdfReader(filepath);
            }
            else
            {
                reader = new PdfReader(filepath, Encoding.Default.GetBytes(password));
            }

            PdfStamper stamp = new PdfStamper(reader, new FileStream(outputFile, FileMode.Create));

            //3iTextSharp.text.Image img = iTextSharp.text.Image.GetInstance(imageFile);

            iTextSharp.text.Image img = iTextSharp.text.Image.GetInstance(imgR, BaseColor.WHITE);

            PdfContentByte waterMark;

            for (int page = 1; page <= reader.NumberOfPages; page++)
            {
                Rectangle realPageSize = reader.GetPageSizeWithRotation(page);

                int width = (int)img.Width;
                int height =(int)img.Height;                

                if (position == "Middle Center")
                {
                    var x = (realPageSize.Left + realPageSize.Right) / 2;
                    var y = (realPageSize.Bottom + realPageSize.Top) / 2;

                    img.SetAbsolutePosition(x-width/2, y-height/2);
                }
                else if (position == "Middle Left")
                {
                    var x = realPageSize.Left + 12;

                    var y = (realPageSize.Bottom + realPageSize.Top) / 2;

                    img.SetAbsolutePosition(x, y - height / 2);
                }
                else if (position == "Middle Right")
                {
                    var x = realPageSize.Right - 12;

                    var y = (realPageSize.Bottom + realPageSize.Top) / 2;

                    img.SetAbsolutePosition(x - width, y - height/2);
                }
                else if (position == "Top Left")
                {
                    var x = realPageSize.Left + 12;

                    var y = realPageSize.Top - 24;

                    img.SetAbsolutePosition(x, y-height);
                }
                else if (position == "Top Right")
                {
                    var x = realPageSize.Right - 12;

                    var y = realPageSize.Top - 24;

                    img.SetAbsolutePosition(x - width, y-height);
                }
                else if (position == "Top Center")
                {
                    var x = (realPageSize.Left + realPageSize.Right) / 2;

                    var y = realPageSize.Top - 24;

                    img.SetAbsolutePosition(x - width / 2, y - height);

                }
                else if (position == "Bottom Left")
                {
                    var x = realPageSize.Left + 12;

                    var y = realPageSize.Bottom + 12;

                    img.SetAbsolutePosition(x, y);
                }
                else if (position == "Bottom Right")
                {
                    var x = realPageSize.Right - 12;

                    var y = realPageSize.Bottom + 12;

                    img.SetAbsolutePosition(x - width, y);
                }
                else if (position == "Bottom Center")
                {
                    var x = (realPageSize.Left + realPageSize.Right) / 2;

                    var y = realPageSize.Bottom + 12;

                    img.SetAbsolutePosition(x - width / 2, y);
                }

                waterMark = stamp.GetOverContent(page);
                waterMark.AddImage(img);
            }

            stamp.FormFlattening = true;
            stamp.Close();

            reader.Close();
            reader.Dispose();
            reader = null;                        
        }
    }
}
