﻿using System;
using System.Collections.Generic;
using System.Text;
using iTextSharp.text.pdf;
using iTextSharp.text.pdf.parser;
using iTextSharp.text;
using System.Data;
using System.IO;

namespace PDFUtilitiesNew
{
    class PageNumbersCreator
    {
        public static string SetPageNumbers(string filepath,string outputFilepath,string password,
            int fromPage, int toPage,int numberFormat,string customFormat,string position)
        {
            try
            {
                PdfReader reader = null;

                if (String.IsNullOrEmpty(password))
                {
                    reader = new PdfReader(filepath);
                }
                else
                {
                    reader = new PdfReader(filepath, Encoding.Default.GetBytes(password));
                }

                string watermarkText = "";

                if (numberFormat == 0)
                {
                    watermarkText="Page [N]";
                }
                else if (numberFormat == 1)
                {
                    watermarkText = "Page [N] of [M]";
                }
                else if (numberFormat == 2)
                {
                    watermarkText = "[N]";
                }
                else if (numberFormat == 3)
                {
                    watermarkText = "[N] of [M]";
                }
                else if (numberFormat == 4)
                {
                    watermarkText = "[RN]";
                }
                else if (numberFormat == 5)
                {
                    watermarkText = "[RN] of [RM]";
                }
                else if (numberFormat == 6)
                {
                    watermarkText = customFormat;
                }

                if (fromPage == -1)
                {
                    fromPage = 1;
                }

                using (var ms = new MemoryStream(10 * 1024))
                {
                    using (var stamper = new PdfStamper(reader, ms))
                    {
                        for (var i = 1; i <= reader.NumberOfPages; i++)
                        {
                            if (toPage!=-1 && toPage<i)
                            {
                                break;
                            }

                            if (((fromPage == 0) || (i >= fromPage)) && ((toPage == -1) || (i <= toPage)))
                            {
                                string pagenumber = watermarkText.Replace("[N]", (i-fromPage+1).ToString()).Replace("[M]", reader.NumberOfPages.ToString())
                                    .Replace("[RN]", ToRoman((i-fromPage+1))).Replace("[M]", ToRoman(reader.NumberOfPages));

                                var dc = stamper.GetOverContent(i);
                                AddWaterMarkText(
                                    dc,
                                    pagenumber,
                                    BaseFont.CreateFont(BaseFont.HELVETICA, Encoding.ASCII.EncodingName, false),
                                    14,
                                    0,
                                    new BaseColor(0, 0, 0), position,
                                    reader.GetPageSizeWithRotation(i));
                            }
                        }
                        stamper.Close();
                    }

                    System.IO.File.WriteAllBytes(outputFilepath, ms.ToArray());
                }

                reader.Close();
                reader.Dispose();
                reader = null;                

                return "";               
                
            }

            catch (Exception ex)
            {
                return "Error could not add Page Numbers of document " + System.IO.Path.GetFileName(filepath) + " " + ex.Message;
            }
        }

        /// <summary>
        /// This method adds watermark text under pdf content
        /// </summary>
        /// <param name="pdfData">pdf content bytes</param>
        /// <param name="watermarkText">text to be shown as watermark</param>
        /// <param name="font">base font</param>
        /// <param name="fontSize">font sieze</param>
        /// <param name="angle">angle at which watermark needs to be shown</param>
        /// <param name="color">water mark color</param>
        /// <param name="realPageSize">pdf page size</param>
        private static void AddWaterMarkText(PdfContentByte pdfData, string watermarkText, BaseFont font, float fontSize, float angle, BaseColor color, string position, Rectangle realPageSize)
        {
            double radians = (angle * Math.PI) / 180;

            var gstate = new PdfGState { FillOpacity = 0.35f, StrokeOpacity = 0.3f };
            pdfData.SaveState();
            pdfData.SetGState(gstate);
            pdfData.SetColorFill(color);
            pdfData.BeginText();
            pdfData.SetFontAndSize(font, fontSize);

            float width = font.GetWidthPoint(watermarkText, fontSize);
            float height = font.GetAscentPoint(watermarkText, fontSize) - font.GetDescentPoint(watermarkText, fontSize);

            if (position == "Middle Center")
            {
                var x = (realPageSize.Left + realPageSize.Right) / 2;
                var y = (realPageSize.Bottom + realPageSize.Top) / 2;

                pdfData.ShowTextAligned(Element.ALIGN_CENTER, watermarkText, x, y, angle);
            }
            else if (position == "Middle Left")
            {
                //var x = realPageSize.Left + width / 2;

                var x = realPageSize.Left + (float)(Math.Cos(radians) * width) / 2;

                var y = (realPageSize.Bottom + realPageSize.Top) / 2;

                pdfData.ShowTextAligned(Element.ALIGN_CENTER, watermarkText, x, y, angle);
            }
            else if (position == "Middle Right")
            {
                //var x = realPageSize.Right - width / 2;

                var x = realPageSize.Right - (float)(Math.Cos(radians) * width) / 2;

                var y = (realPageSize.Bottom + realPageSize.Top) / 2;

                pdfData.ShowTextAligned(Element.ALIGN_CENTER, watermarkText, x, y, angle);
            }
            else if (position == "Top Left")
            {
                //var x = realPageSize.Left + width/2;

                //var y = realPageSize.Top - height;

                var x = realPageSize.Left + (float)(Math.Cos(radians) * width) / 2+12;

                var y = realPageSize.Top - (float)(Math.Sin(radians) * width) / 2-24;

                pdfData.ShowTextAligned(Element.ALIGN_CENTER, watermarkText, x, y, angle);
            }
            else if (position == "Top Right")
            {
                //var x = realPageSize.Right - width/2;

                //var y = realPageSize.Top - height;

                var x = realPageSize.Right - (float)(Math.Cos(radians) * width) / 2-12;

                var y = realPageSize.Top - (float)(Math.Sin(radians) * width) / 2-24;

                pdfData.ShowTextAligned(Element.ALIGN_CENTER, watermarkText, x, y, angle);
            }
            else if (position == "Top Center")
            {
                var x = (realPageSize.Left + realPageSize.Right) / 2;

                var y = realPageSize.Top - (float)(Math.Sin(radians) * width) / 2-24;

                //var y = realPageSize.Top - height;

                pdfData.ShowTextAligned(Element.ALIGN_CENTER, watermarkText, x, y, angle);

            }
            else if (position == "Bottom Left")
            {

                //var x = realPageSize.Left + width / 2;

                var x = realPageSize.Left + (float)(Math.Cos(radians) * width) / 2+12;

                //var y = realPageSize.Bottom + height/2;

                var y = realPageSize.Bottom + (float)(Math.Sin(radians) * width) / 2+12;

                pdfData.ShowTextAligned(Element.ALIGN_CENTER, watermarkText, x, y, angle);
            }
            else if (position == "Bottom Right")
            {
                //var x = realPageSize.Right - width / 2;

                //var y = realPageSize.Bottom + height / 2;

                var x = realPageSize.Right - (float)(Math.Cos(radians) * width) / 2-12;

                var y = realPageSize.Bottom + (float)(Math.Sin(radians) * width) / 2+12;

                pdfData.ShowTextAligned(Element.ALIGN_CENTER, watermarkText, x, y, angle);
            }
            else if (position == "Bottom Center")
            {
                var x = (realPageSize.Left + realPageSize.Right) / 2;

                //var y = realPageSize.Bottom + height / 2;                

                var y = realPageSize.Bottom + (float)(Math.Sin(radians) * width) / 2+12;

                pdfData.ShowTextAligned(Element.ALIGN_CENTER, watermarkText, x, y, angle);

            }

            pdfData.EndText();
            pdfData.RestoreState();
        }

        public static string ToRoman(int number)
        {
            if ((number < 0) || (number > 3999)) throw new ArgumentOutOfRangeException("insert value betwheen 1 and 3999");
            if (number < 1) return string.Empty;
            if (number >= 1000) return "M" + ToRoman(number - 1000);
            if (number >= 900) return "CM" + ToRoman(number - 900);
            if (number >= 500) return "D" + ToRoman(number - 500);
            if (number >= 400) return "CD" + ToRoman(number - 400);
            if (number >= 100) return "C" + ToRoman(number - 100);
            if (number >= 90) return "XC" + ToRoman(number - 90);
            if (number >= 50) return "L" + ToRoman(number - 50);
            if (number >= 40) return "XL" + ToRoman(number - 40);
            if (number >= 10) return "X" + ToRoman(number - 10);
            if (number >= 9) return "IX" + ToRoman(number - 9);
            if (number >= 5) return "V" + ToRoman(number - 5);
            if (number >= 4) return "IV" + ToRoman(number - 4);
            if (number >= 1) return "I" + ToRoman(number - 1);
            throw new ArgumentOutOfRangeException("something bad happened");
        }
    }
}
